@@ -1,35 +1,46 @@
 /* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
  *
- * The contents of this file are subject to the Mozilla Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/MPL/
+ * ***** BEGIN LICENSE BLOCK *****
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
  *
- * The Original Code is JSIRC Library
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
- * The Initial Developer of the Original Code is New Dimensions Consulting,
- * Inc. Portions created by New Dimensions Consulting, Inc. are
- * Copyright (C) 1999 New Dimenstions Consulting, Inc. All
- * Rights Reserved.
+ * The Original Code is JSIRC Library.
+ *
+ * The Initial Developer of the Original Code is
+ * New Dimensions Consulting, Inc.
+ * Portions created by the Initial Developer are Copyright (C) 1999
+ * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
- *  Robert Ginda, rginda@ndcico.com, original author
+ *   Robert Ginda, rginda@ndcico.com, original author
  *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the MPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the MPL, the GPL or the LGPL.
  *
- * JavaScript utility functions.
- *
- * 1999-08-15 rginda@ndcico.com           v1.0
- */
+ * ***** END LICENSE BLOCK ***** */
 
 var utils = new Object();
 
 var DEBUG = true;
-var dd;
+var dd, warn, TEST, ASSERT;
 
 if (DEBUG) {
     var _dd_pfx = "";
@@ -62,7 +73,7 @@ if (DEBUG) {
                      return;
                  _dd_disableDepth = Number.MAX_VALUE;
                  var sufx = (new Date() - _dd_timeStack.pop()) / 1000 + " sec";
-                 _dd_currentIndent = 
+                 _dd_currentIndent =
                      _dd_currentIndent.substr(0, _dd_currentIndent.length -
                                               _dd_indentLength);
                  if (_dd_lastDumpWasOpen)
@@ -78,10 +89,24 @@ if (DEBUG) {
                      dump("\n");
                  dump(_dd_pfx + _dd_currentIndent + str + "\n");
                  _dd_lastDumpWasOpen = false;
-             }    
+             }
          }
+    warn = function (msg) { dd("** WARNING " + msg + " **"); }
+    TEST = ASSERT = function _assert(expr, msg) {
+                 if (!expr) {
+                     var m = "** ASSERTION FAILED: " + msg + " **\n" +
+                             getStackTrace();
+                     try {
+                         alert(m);
+                     } catch(ex) {}
+                     dd(m);
+                     return false;
+                 } else {
+                     return true;
+                 }
+             }
 } else {
-    dd = function (){};
+    dd = warn = TEST = ASSERT = function (){};
 }
 
 var jsenv = new Object();
@@ -94,6 +119,7 @@ jsenv.HAS_RHINO = (typeof defineClass == "function");
 jsenv.HAS_DOCUMENT = (typeof document == "object");
 jsenv.HAS_NSPR_EVENTQ = jsenv.HAS_DOCUMENT;
 jsenv.HAS_STREAM_PROVIDER = ("nsIStreamProvider" in Components.interfaces);
+jsenv.HAS_SERVER_SOCKETS = ("nsIServerSocket" in Components.interfaces);
 
 function dumpObject (o, pfx, sep)
 {
@@ -101,7 +127,7 @@ function dumpObject (o, pfx, sep)
     var s = "";
 
     sep = (typeof sep == "undefined") ? " = " : sep;
-    pfx = (typeof pfx == "undefined") ? "" : pfx;    
+    pfx = (typeof pfx == "undefined") ? "" : pfx;
 
     for (p in o)
     {
@@ -143,7 +169,7 @@ function dumpObjectTree (o, recurse, compress, level)
         level = 0;
     if (typeof compress == "undefined")
         compress = true;
-    
+
     for (var i = 0; i < level; i++)
         pfx += (compress) ? "| " : "|  ";
 
@@ -152,7 +178,7 @@ function dumpObjectTree (o, recurse, compress, level)
     for (i in o)
     {
         var t, ex;
-        
+
         try
         {
             t = typeof o[i];
@@ -161,7 +187,7 @@ function dumpObjectTree (o, recurse, compress, level)
         {
             t = "ERROR";
         }
-        
+
         switch (t)
         {
             case "function":
@@ -183,9 +209,9 @@ function dumpObjectTree (o, recurse, compress, level)
                     s += " null\n";
                     break;
                 }
-                
+
                 s += "\n";
-                
+
                 if (!compress)
                     s += pfx + "|\n";
                 if ((i != "parent") && (recurse))
@@ -195,7 +221,7 @@ function dumpObjectTree (o, recurse, compress, level)
 
             case "string":
                 if (o[i].length > 200)
-                    s += pfx + tee + i + " (" + t + ") " + 
+                    s += pfx + tee + i + " (" + t + ") " +
                         o[i].length + " chars\n";
                 else
                     s += pfx + tee + i + " (" + t + ") '" + o[i] + "'\n";
@@ -204,10 +230,10 @@ function dumpObjectTree (o, recurse, compress, level)
             case "ERROR":
                 s += pfx + tee + i + " (" + t + ") ?\n";
                 break;
-                
+
             default:
                 s += pfx + tee + i + " (" + t + ") " + o[i] + "\n";
-                
+
         }
 
         if (!compress)
@@ -216,9 +242,9 @@ function dumpObjectTree (o, recurse, compress, level)
     }
 
     s += pfx + "*\n";
-    
+
     return s;
-    
+
 }
 
 function ecmaEscape(str)
@@ -232,7 +258,7 @@ function ecmaEscape(str)
             rv = "u0" + rv;
         else if (rv.length == 4)
             rv = "u" + rv;
-      
+
         return "%" + rv;
     };
 
@@ -248,7 +274,7 @@ function ecmaUnescape(str)
 {
     function replaceEscapes(seq)
     {
-        var ary = seq.match(/([\da-f]{1,2})(.*)|u([\da-f]{1,4})/);
+        var ary = seq.match(/([\da-f]{1,2})(.*)|u([\da-f]{1,4})/i);
         if (!ary)
             return "<ERROR>";
 
@@ -283,17 +309,17 @@ function replaceVars(str, vars)
         var name = symbol.substr(1);
         if (name in vars)
             return vars[name];
-        
+
         return "$" + name;
     };
-    
+
     return str.replace(/(\$\w[\w\d\-]+)/g, doReplace);
 }
-    
+
 function formatException (ex)
 {
     if (ex instanceof Error)
-        return getMsg (MSG_FMT_JSEXCEPTION, [ex.name, ex.message, ex.fileName, 
+        return getMsg (MSG_FMT_JSEXCEPTION, [ex.name, ex.message, ex.fileName,
                                              ex.lineNumber]);
 
     return String(ex);
@@ -314,20 +340,20 @@ function Clone (obj)
         robj[p] = obj[p];
 
     return robj;
-    
+
 }
 
 function Copy(source, dest, overwrite)
 {
     if (!dest)
         dest = new Object();
-    
+
     for (var p in source)
     {
         if (overwrite || !(p in dest))
             dest[p] = source[p];
     }
-    
+
     return dest;
 }
 
@@ -339,12 +365,12 @@ function Copy(source, dest, overwrite)
 function matchObject (o, pattern, negate)
 {
     negate = Boolean(negate);
-    
+
     function _match (o, pattern)
     {
         if (pattern instanceof Function)
             return pattern(o);
-        
+
         for (p in pattern)
         {
             var val;
@@ -352,12 +378,12 @@ function matchObject (o, pattern, negate)
                  * properties of objects with obj$prop: "foo" syntax      */
                 /*
                   if (p[0] == "$")
-                  val = eval ("o." + 
+                  val = eval ("o." +
                   p.substr(1,p.length).replace (/\$/g, "."));
                   else
                 */
             val = o[p];
-            
+
             if (pattern[p] instanceof Function)
             {
                 if (!pattern[p](val))
@@ -379,32 +405,45 @@ function matchObject (o, pattern, negate)
 
     if (!(pattern instanceof Array))
         return Boolean (negate ^ _match(o, pattern));
-            
+
     for (var i in pattern)
         if (_match (o, pattern[i]))
             return !negate;
 
     return negate;
-    
+
 }
 
-function matchEntry (partialName, list)
+function utils_lcfn(text)
 {
-    
+    return text.toLowerCase();
+}
+
+function matchEntry (partialName, list, lcFn)
+{
+
     if ((typeof partialName == "undefined") ||
         (String(partialName) == ""))
-        return list;
-
-    var ary = new Array();
-
-    for (var i in list)
     {
-        if (list[i].indexOf(partialName) == 0)
-            ary.push (list[i]);
+        var ary = new Array();
+        for (var i in list)
+            ary.push(i);
+        return ary;
+    }
+
+    if (typeof lcFn != "function")
+        lcFn = utils_lcfn;
+
+    ary = new Array();
+
+    for (i in list)
+    {
+        if (lcFn(list[i]).indexOf(lcFn(partialName)) == 0)
+            ary.push(i);
     }
 
     return ary;
-    
+
 }
 
 function encodeChar(ch)
@@ -414,27 +453,30 @@ function encodeChar(ch)
 
 function escapeFileName(fileName)
 {
-    return fileName.replace(/[^\w\d.,#\-_]/g, encodeChar);
+    return fileName.replace(/[^\w\d.,#\-_%]/g, encodeChar);
 }
 
-function getCommonPfx (list)
+function getCommonPfx (list, lcFn)
 {
     var pfx = list[0];
     var l = list.length;
-    
+
+    if (typeof lcFn != "function")
+        lcFn = utils_lcfn;
+
     for (var i = 0; i < l; i++)
     {
         for (var c = 0; c < pfx.length; ++c)
         {
             if (c >= list[i].length)
             {
-                pfx = pfx.substr (0, c);
+                pfx = pfx.substr(0, c);
                 break;
             }
             else
             {
-                if (pfx[c] != list[i][c])
-                    pfx = pfx.substr (0, c);
+                if (lcFn(pfx[c]) != lcFn(list[i][c]))
+                    pfx = pfx.substr(0, c);
             }
         }
     }
@@ -447,7 +489,7 @@ function openTopWin (url)
 {
     return openDialog (getBrowserURL(), "_blank", "chrome,all,dialog=no", url);
 }
-    
+
 function getWindowByType (windowType)
 {
     const MEDIATOR_CONTRACTID =
@@ -465,10 +507,10 @@ function renameProperty (obj, oldname, newname)
 
     if (oldname == newname)
         return;
-    
+
     obj[newname] = obj[oldname];
     delete obj[oldname];
-    
+
 }
 
 function newObject(contractID, iface)
@@ -476,17 +518,24 @@ function newObject(contractID, iface)
     if (!jsenv.HAS_XPCOM)
         return null;
 
-    var obj = Components.classes[contractID].createInstance();
     var rv;
+    var cls = Components.classes[contractID];
+    
+    if (!cls)
+        return null;
 
     switch (typeof iface)
     {
+        case "undefined":
+            rv = cls.createInstance();
+            break;
+
         case "string":
-            rv = obj.QueryInterface(Components.interfaces[iface]);
+            rv = cls.createInstance(Components.interfaces[iface]);
             break;
 
         case "object":
-            rv = obj.QueryInterface[iface];
+            rv = cls.createInstance(iface);
             break;
 
         default:
@@ -495,7 +544,41 @@ function newObject(contractID, iface)
     }
 
     return rv;
+
+}
+
+function getService(contractID, iface)
+{
+    if (!jsenv.HAS_XPCOM)
+        return null;
+
+    var rv;
+    var cls = Components.classes[contractID];
     
+    if (!cls)
+        return null;
+
+    switch (typeof iface)
+    {
+        case "undefined":
+            rv = cls.getService();
+            break;
+
+        case "string":
+            rv = cls.getService(Components.interfaces[iface]);
+            break;
+
+        case "object":
+            rv = cls.getService(iface);
+            break;
+
+        default:
+            rv = null;
+            break;
+    }
+
+    return rv;
+
 }
 
 function getContentWindow(frame)
@@ -530,30 +613,30 @@ function getPriv (priv)
         dd ("getPriv: unable to get privlege '" + priv + "': " + e);
         rv = false;
     }
-    
+
     return rv;
-    
+
 }
 
 function len(o)
 {
     var l = 0;
-    
+
     for (var p in o)
         ++l;
-    
+
     return l;
 }
 
 function keys (o)
 {
     var rv = new Array();
-    
+
     for (var p in o)
         rv.push(p);
 
     return rv;
-    
+
 }
 
 function stringTrim (s)
@@ -562,7 +645,7 @@ function stringTrim (s)
         return "";
     s = s.replace (/^\s+/, "");
     return s.replace (/\s+$/, "");
-    
+
 }
 
 /* the offset should be in seconds, it will be rounded to 2 decimal places */
@@ -596,7 +679,7 @@ function formatDateOffset (offset, format)
         format = format.replace ("%m", minutes);
         format = format.replace ("%s", seconds);
     }
-    
+
     return format;
 }
 
@@ -618,7 +701,7 @@ function arrayIndexOf (ary, elem)
 
     return -1;
 }
-    
+
 function arrayInsertAt (ary, i, o)
 {
     ary.splice (i, 0, o);
@@ -677,10 +760,10 @@ function splitLongWord (str, pos)
 
     var ary = new Array();
     var right = str;
-    
+
     while (right.length > pos)
     {
-        /* search for a nice place to break the word, fuzzfactor of +/-5, 
+        /* search for a nice place to break the word, fuzzfactor of +/-5,
          * centered around |pos| */
         var splitPos =
             right.substring(pos - 5, pos + 5).search(/[^A-Za-z0-9]/);
@@ -705,7 +788,7 @@ function getRandomElement (ary)
 function roundTo (num, prec)
 {
 
-    return Math.round(num * Math.pow (10, prec)) / Math.pow (10, prec);   
+    return Math.round(num * Math.pow (10, prec)) / Math.pow (10, prec);
 
 }
 
@@ -739,7 +822,7 @@ function getStackTrace ()
     }
 
     return str;
-    
+
 }
 
 function getInterfaces (cls)
@@ -765,7 +848,7 @@ function getInterfaces (cls)
     }
 
     return rv;
-    
+
 }
 
 /**
@@ -775,10 +858,10 @@ function getInterfaces (cls)
  * @param ary           an array of objects
  * @param func_name     string name of function to call.
  * @param data          data object to pass to each object.
- */      
+ */
 function mapObjFunc(ary, func_name, data)
 {
-    /* 
+    /*
      * WARNING: Caller assumes resonsibility to verify ary
      * and func_name
      */
@@ -821,10 +904,27 @@ function getSpecialDirectory(name)
 
 function getFileFromURLSpec(url)
 {
-    const FILE_CTRID = "@mozilla.org/network/protocol;1?name=file";
     const nsIFileProtocolHandler = Components.interfaces.nsIFileProtocolHandler;
-    
-    var handler = Components.classes[FILE_CTRID].createInstance();
+
+    var service = getService("@mozilla.org/network/io-service;1", 
+                             "nsIIOService");
+
+    /* In sept 2002, bug 166792 moved this method to the nsIFileProtocolHandler
+     * interface, but we need to support older versions too. */
+    if ("getFileFromURLSpec" in service)
+        return service.getFileFromURLSpec(url);
+
+    /* In builds before 2002-08-15, there is no getFileFromURLSpec at all.
+     * Instead, we have nsIIOservice.initFileFromURLSpec(nsIFile, string).
+     */
+    if ("initFileFromURLSpec" in service)
+    {
+        var file = newObject("@mozilla.org/file/local;1", "nsILocalFile");
+        service.initFileFromURLSpec(file, url);
+        return file;
+    }
+
+    var handler = service.getProtocolHandler("file");
     handler = handler.QueryInterface(nsIFileProtocolHandler);
     return handler.getFileFromURLSpec(url);
 }
@@ -839,7 +939,7 @@ function getURLSpecFromFile (file)
 
     const nsIIOService = Components.interfaces.nsIIOService;
     const nsILocalFile = Components.interfaces.nsILocalFile;
-    
+
     if (typeof file == "string")
     {
         var fileObj =
@@ -847,7 +947,7 @@ function getURLSpecFromFile (file)
         fileObj.initWithPath(file);
         file = fileObj;
     }
-    
+
     var service = Components.classes[IOS_CTRID].getService(nsIIOService);
     /* In sept 2002, bug 166792 moved this method to the nsIFileProtocolHandler
      * interface, but we need to support older versions too. */
@@ -884,6 +984,66 @@ function confirm(msg, parent, title)
     return ps.confirm (parent, title, msg);
 }
 
+function confirmEx(msg, buttons, defaultButton, checkText, 
+                   checkVal, parent, title)
+{
+    /* Note that on versions before Mozilla 0.9, using 3 buttons,
+     * the revert or dontsave button, or custom button titles will NOT work.
+     *
+     * The buttons should be listed in the 'accept', 'cancel' and 'extra' order,
+     * and the exact button order is host app- and platform-dependant.
+     * For example, on Windows this is usually [button 1] [button 3] [button 2],
+     * and on Linux [button 3] [button 2] [button 1]. 
+     */
+    var PROMPT_CTRID = "@mozilla.org/embedcomp/prompt-service;1";
+    var nsIPromptService = Components.interfaces.nsIPromptService;
+    var ps = Components.classes[PROMPT_CTRID].getService(nsIPromptService);
+
+    var buttonConstants = {
+        ok: ps.BUTTON_TITLE_OK,
+        cancel: ps.BUTTON_TITLE_CANCEL,
+        yes: ps.BUTTON_TITLE_YES,
+        no: ps.BUTTON_TITLE_NO,
+        save: ps.BUTTON_TITLE_SAVE,
+        revert: ps.BUTTON_TITLE_REVERT,
+        dontsave: ps.BUTTON_TITLE_DONT_SAVE
+    };
+    var buttonFlags = 0;
+    var buttonText = [null, null, null];
+
+    if (!isinstance(buttons, Array)) 
+        throw "buttons parameter must be an Array";
+    if ((buttons.length < 1) || (buttons.length > 3))
+        throw "the buttons array must have 1, 2 or 3 elements";
+
+    for (var i = 0; i < buttons.length; i++)
+    {
+        var buttonFlag = ps.BUTTON_TITLE_IS_STRING;
+        if ((buttons[i][0] == "!") && (buttons[i].substr(1) in buttonConstants))
+            buttonFlag = buttonConstants[buttons[i].substr(1)];
+        else
+            buttonText[i] = buttons[i];
+
+        buttonFlags += ps["BUTTON_POS_" + i] * buttonFlag;
+    }
+
+    // ignore anything but a proper number
+    var defaultIsNumber = (typeof defaultButton == "number");
+    if (defaultIsNumber && arrayHasElementAt(buttons, defaultButton))
+        buttonFlags += ps["BUTTON_POS_" + defaultButton + "_DEFAULT"];
+
+    if (!parent)
+        parent = window;
+    if (!title)
+        title = MSG_CONFIRM;
+    if (!checkVal)
+        checkVal = new Object();
+
+    rv = ps.confirmEx(parent, title, msg, buttonFlags, buttonText[0], 
+                      buttonText[1], buttonText[2], checkText, checkVal);
+    return rv;
+}
+
 function prompt(msg, initial, parent, title)
 {
     var PROMPT_CTRID = "@mozilla.org/embedcomp/prompt-service;1";
@@ -918,6 +1078,15 @@ function promptPassword(msg, initial, parent, title)
     return rv.value;
 }
 
+function viewCert(cert, parent)
+{
+    var cd = getService("@mozilla.org/nsCertificateDialogs;1",
+                        "nsICertificateDialogs");
+    if (!parent)
+        parent = window;
+    cd.viewCert(parent, cert);
+}
+
 function getHostmaskParts(hostmask)
 {
     var rv;
@@ -945,3 +1114,14 @@ function getHostmaskParts(hostmask)
     rv.mask = rv.nick + "!" + rv.user + "@" + rv.host;
     return rv;
 }
+
+function isinstance(inst, base)
+{
+    /* Returns |true| if |inst| was constructed by |base|. Not 100% accurate,
+     * but plenty good enough for us. This is to work around the fix for bug
+     * 254067 which makes instanceof fail if the two sides are 'from'
+     * different windows (something we don't care about).
+     */
+    return (inst && inst.constructor && base &&
+            (inst.constructor.name == base.name));
+}
