@@ -1,25 +1,42 @@
-# The contents of this file are subject to the Netscape Public
-# License Version 1.1 (the "License"); you may not use this file
-# except in compliance with the License. You may obtain a copy of
-# the License at http://www.mozilla.org/NPL/
-# 
-# Software distributed under the License is distributed on an "AS
-# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
-# implied. See the License for the specific language governing
-# rights and limitations under the License.
-# 
+# ***** BEGIN LICENSE BLOCK *****
+# Version: MPL 1.1/GPL 2.0/LGPL 2.1
+#
+# The contents of this file are subject to the Mozilla Public License Version
+# 1.1 (the "License"); you may not use this file except in compliance with
+# the License. You may obtain a copy of the License at
+# http://www.mozilla.org/MPL/
+#
+# Software distributed under the License is distributed on an "AS IS" basis,
+# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+# for the specific language governing rights and limitations under the
+# License.
+#
 # The Original Code is Mozilla Communicator client code, released
 # March 31, 1998.
-# 
-# The Initial Developer of the Original Code is David Hyatt. 
-# Portions created by David Hyatt are
-# Copyright (C) 2002 David Hyatt. All
-# Rights Reserved.
-# 
-# Contributor(s): 
+#
+# The Initial Developer of the Original Code is
+# David Hyatt.
+# Portions created by the Initial Developer are Copyright (C) 2002
+# the Initial Developer. All Rights Reserved.
+#
+# Contributor(s):
 #   David Hyatt (hyatt@apple.com)
 #   Blake Ross (blaker@netscape.com)
 #   Joe Hewitt (hewitt@netscape.com)
+#
+# Alternatively, the contents of this file may be used under the terms of
+# either the GNU General Public License Version 2 or later (the "GPL"), or
+# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+# in which case the provisions of the GPL or the LGPL are applicable instead
+# of those above. If you wish to allow use of your version of this file only
+# under the terms of either the GPL or the LGPL, and not to allow others to
+# use your version of this file under the terms of the MPL, indicate your
+# decision by deleting the provisions above and replace them with the notice
+# and other provisions required by the GPL or the LGPL. If you do not delete
+# the provisions above, a recipient may use your version of this file under
+# the terms of any one of the MPL, the GPL or the LGPL.
+#
+# ***** END LICENSE BLOCK *****
 
 const kRowMax = 4;
 const kWindowWidth = 600;
@@ -533,10 +550,11 @@ function setWrapperType(aItem, aWrapper)
 function setDragActive(aItem, aValue)
 {
   var node = aItem;
-  var value = "left";
+  var direction = window.getComputedStyle(aItem, null).direction;
+  var value = direction == "ltr"? "left" : "right";
   if (aItem.localName == "toolbar") {
     node = aItem.lastChild;
-    value = "right";
+    value = direction == "ltr"? "right" : "left";
   }
   
   if (!node)
@@ -733,11 +751,16 @@ var toolbarDNDObserver =
     if (dropTarget.localName == "toolbar") {
       gCurrentDragOverItem = dropTarget;
     } else {
-      var dropTargetWidth = dropTarget.boxObject.width;
-      var dropTargetX = dropTarget.boxObject.x;
-
       gCurrentDragOverItem = null;
-      if (aEvent.clientX > (dropTargetX + (dropTargetWidth / 2))) {
+
+      var direction = window.getComputedStyle(dropTarget.parentNode, null).direction;
+      var dropTargetCenter = dropTarget.boxObject.x + (dropTarget.boxObject.width / 2);
+      if (direction == "ltr")
+        dragAfter = aEvent.clientX > dropTargetCenter;
+      else
+        dragAfter = aEvent.clientX < dropTargetCenter;
+        
+      if (dragAfter) {
         gCurrentDragOverItem = dropTarget.nextSibling;
         if (!gCurrentDragOverItem)
           gCurrentDragOverItem = toolbar;
