@@ -2172,7 +2172,9 @@ var gMessageNotificationBar =
       isJunk = ((junkScore != "") && (junkScore != "0"));
     }
 
-    this.updateMsgNotificationBar (isJunk ? kMsgNotificationJunkBar : kMsgNotificationNoStatus);
+    // phishing scams take precedence over junk
+    if (this.mMsgNotificationBar.selectedIndex != kMsgNotificationPhishingBar && isJunk)
+      this.updateMsgNotificationBar (kMsgNotificationJunkBar);
 
     goUpdateCommand('button_junk');
 
@@ -2181,9 +2183,8 @@ var gMessageNotificationBar =
 
   setRemoteContentMsg: function (aMsgHdr)
   {  
-    // The phishing message and junk message takes precedence over the remote content msg
-    if (this.mMsgNotificationBar.selectedIndex != kMsgNotificationJunkBar && 
-        this.mMsgNotificationBar.selectedIndex != kMsgNotificationPhishingBar)
+    // phishing and junk messages take precedence over the remote content msg...
+    if (this.mMsgNotificationBar.selectedIndex == kMsgNotificationNoStatus)
       this.updateMsgNotificationBar(aMsgHdr && aMsgHdr.getUint32Property("remoteContentPolicy") == kBlockRemoteContent ? 
                                     kMsgNotificationRemoteImages : kMsgNotificationNoStatus);
   },
@@ -2201,9 +2202,8 @@ var gMessageNotificationBar =
         return; 
     }
 
-    // The Junk message takes precedence over the phishing message...so skip this step
-    // if the message is already marked as junk
-    if (this.mMsgNotificationBar.selectedIndex != kMsgNotificationJunkBar && isMsgEmailScam(aUrl))
+    // phishing scams take precedence over junk and remote images.
+    if (isMsgEmailScam(aUrl))
       this.updateMsgNotificationBar(kMsgNotificationPhishingBar);
   },
 
@@ -2305,7 +2305,12 @@ function OnMsgLoaded(aUrl)
     // if the user clicks on another message then that message stays selected
     // and the selection does not "snap back" to the message chosen by
     // SetNextMessageAfterDelete() when the operation completes (bug 243532).
-    gNextMessageViewIndexAfterDelete = -2;
+    // But the just loaded message might be getting deleted, if the user
+    // deletes it before the message is loaded (bug 183394)
+
+    var wintype = document.documentElement.getAttribute('windowtype');
+    if (wintype == "mail:messageWindow" || GetThreadTree().view.selection.currentIndex != gSelectedIndexWhenDeleting)
+      gNextMessageViewIndexAfterDelete = -2;
 
     if (!(/type=application\/x-message-display/.test(msgURI)))
       msgHdr = messenger.messageServiceFromURI(msgURI).messageURIToMsgHdr(msgURI);
@@ -2319,7 +2324,6 @@ function OnMsgLoaded(aUrl)
 
     if (msgHdr && !msgHdr.isRead)
     {
-      var wintype = document.firstChild.getAttribute('windowtype');
       if (markReadOnADelay && wintype == "mail:3pane") // only use the timer if viewing using the 3-pane preview pane and the user has set the pref
       {
         ClearPendingReadTimer();
